# cli_parser.py

from .errors import ErrorCause, BitriseReportsError
from dateutil.parser import parse


def validated_app(app_name):
    if app_name:
        return app_name

    cause = ErrorCause.EntrypointHandling
    message = "Missing bitrise app name"
    raise BitriseReportsError(cause, message)


def validated_date(date, include_hours=False):

    time = "23:59:59" if include_hours else "00:00:00"
    iso_datetime = f"{date}T{time}"
    try:
        return parse(iso_datetime)
    except:
        cause = ErrorCause.EntrypointHandling
        message = f"Cannot convert date time -> {iso_datetime}"
        raise BitriseReportsError(cause, message)


def validated_report(report):

    trimmed = report.strip()

    if trimmed in ["stdout", "json", "excel"]:
        return trimmed

    cause = ErrorCause.EntrypointHandling
    message = f"Unsupported report type -> {report}. Options: stdout | json | excel"
    raise BitriseReportsError(cause, message)



# models.py

from dataclasses import dataclass
from enum import Enum
from typing import Dict
from datetime import datetime


class MachineSize(Enum):
    g1small = "standard"
    g1medium = "elite"
    g1large = "elite-xl"
    g2small = "g2.4core"
    g2medium = "g2.8core"
    g2large = "g2.12core"


class BuildStack(Enum):
    linux = "linux"
    osx = "macos"


class ExecutionStatus(Enum):
    success = 1
    error = 2
    other = 3


@dataclass(frozen=True)
class BuildMachine:
    id: str
    size: MachineSize
    stack: BuildStack


@dataclass(frozen=True)
class BitriseProject:
    id: str
    slug: str


@dataclass(frozen=True)
class BitriseWorkflow:
    id: str


@dataclass(frozen=True)
class BuildMinutes:
    queued: int
    building: int
    total: int


@dataclass(frozen=True)
class BitriseBuild:
    project: BitriseProject
    machine: BuildMachine
    workflow: str
    minutes: BuildMinutes
    status: ExecutionStatus


@dataclass(frozen=True)
class CrunchedNumbers:
    count: int
    queued: int
    building: int
    total: int
    successes: int = None
    failures: int = None
    credits: int = None


@dataclass(frozen=True)
class BitriseBreakdown:
    name: str
    details: Dict


@dataclass(frozen=True)
class EvaluationCriteria:
    bitrise_app: str
    starting_at: datetime
    ending_at: datetime



from . import entrypoint

import pretty_errors


def main(argv=None):

    pretty_errors.configure(
        separator_character="*",
        filename_display=pretty_errors.FILENAME_EXTENDED,
        line_number_first=True,
        display_link=True,
        lines_before=5,
        lines_after=2,
        line_color=pretty_errors.RED + ">  " + pretty_errors.default_config.line_color,
        code_color=" " + pretty_errors.default_config.line_color,
        truncate_code=True,
        display_locals=True,
    )

    entrypoint.launch()



# metrics.py

from .models import (
    BitriseBreakdown,
    BuildMinutes,
    BuildStack,
    CrunchedNumbers,
    ExecutionStatus,
    MachineSize,
)

from itertools import groupby

LINUX_CREDITS_MULTIPLIER = {
    MachineSize.g1small: 1,
    MachineSize.g1medium: 2,
    MachineSize.g1large: 4,
}

OSX_CREDITS_MULTIPLIER = {
    MachineSize.g1small: 2,
    MachineSize.g1medium: 4,
    MachineSize.g2small: 2,
    MachineSize.g2medium: 4,
    MachineSize.g2large: 6,
}


class MetricsCruncher(object):
    def breakdown_per_project(self, builds):
        count, summary = self.__breakdown_builds(builds, lambda k: k.project)
        return BitriseBreakdown("Project Numbers", summary)

    def breakdown_per_machine(self, builds):
        count, summary = self.__breakdown_builds(builds, lambda k: k.machine)
        return BitriseBreakdown("Per machine", summary)

    def breakdown_per_workflow(self, builds):
        count, summary = self.__breakdown_builds(builds, lambda k: k.workflow)
        return BitriseBreakdown("Per Workflow", summary)

    def __breakdown_builds(self, builds, criteria):
        count = 0
        summary = {}

        for key, grouped in groupby(builds, criteria):
            total, minutes, successes, failures, credits = self.__analyse(grouped)

            if key not in summary.keys():
                numbers = CrunchedNumbers(0, 0, 0, 0, 0, 0, 0)
                summary[key] = numbers

            actual = summary[key]

            updated = CrunchedNumbers(
                count=actual.count + total,
                queued=actual.queued + minutes.queued,
                building=actual.building + minutes.building,
                total=actual.total + minutes.total,
                successes=actual.successes + successes,
                failures=actual.failures + failures,
                credits=actual.credits + credits,
            )

            summary[key] = updated
            count = count + total

        return count, summary

    def __analyse(self, builds):
        count = 0
        minutes = BuildMinutes(0, 0, 0)
        successes = 0
        failures = 0
        credits = 0

        for build in builds:
            count = count + 1
            minutes = self.__sum_minutes(minutes, build.minutes)
            successes = successes + 1 if build.status == ExecutionStatus.success else successes
            failures = failures + 1 if build.status == ExecutionStatus.error else failures
            credits = credits + self.__compute_credits(build)

        return [count, minutes, successes, failures, credits]

    def __sum_minutes(self, target, another):
        return BuildMinutes(
            target.queued + another.queued,
            target.building + another.building,
            target.total + another.total,
        )

    def __compute_credits(self, build):
        mac = build.machine.stack == BuildStack.osx
        multiplier = OSX_CREDITS_MULTIPLIER if mac else LINUX_CREDITS_MULTIPLIER
        return multiplier[build.machine.size] * build.minutes.total



# errors.py

from enum import Enum


class BitriseReportsError(RuntimeError):
    def __init__(self, cause, message):
        self.cause = cause
        self.message = message


class ErrorCause(Enum):
    NetworkingInfrastructure = 1
    DataConversion = 2
    MetricsExtraction = 3
    MiddlewareOrchestration = 4
    EntrypointHandling = 5



# app.py


from . import di

import click
import logging
import sys

APP_HELP = "The title of your app in Bitrise"
ENDING_HELP = "Ending date to drive the analysis (YYYY-MM-DD)"
START_HELP = "Starting date to drive the analysis (YYYY-MM-DD)"
BITRISE_PAT_HELP = "A Personal Access Token (PAT) for Bitrise API"
VELOCITY_HELP = "Estimate Bitrise Velocity credits related to builds"
STATUSES_HELP = "Estimate successes and failures related to builds"
OUTPUT_HELP = "The style of report you want (stdout | json | excel). Defaults to stdout"

logging.basicConfig(level=logging.INFO)


@click.command()
@click.option("--token", required=True, help=BITRISE_PAT_HELP)
@click.option("--app", required=True, help=APP_HELP)
@click.option("--starting", required=True, help=START_HELP)
@click.option("--ending", required=True, help=ENDING_HELP)
@click.option("--report", required=False, default="stdout", help=OUTPUT_HELP)
@click.option("--velocity", is_flag=True, default=False, help=VELOCITY_HELP)
@click.option("--statuses", is_flag=True, default=False, help=STATUSES_HELP)
def launch(token, app, starting, ending, velocity, statuses, report):
    app = di.inject(token, app, starting, ending, velocity, statuses, report)
    app.execute()
    sys.exit(0)



# bitrise.py

from .errors import ErrorCause, BitriseReportsError
from .models import BitriseBuild, BitriseProject, BuildStack, ExecutionStatus
from .models import BuildMachine, BuildMinutes, BitriseWorkflow, MachineSize

from datetime import datetime
from math import ceil

import requests
import time

BITRISE_API_URL = "https://api.bitrise.io/v0.1"
FIRST_PAGE = "first-page"
NO_MORE_PAGES = "no-more-pages"


class Bitrise(object):
    def __init__(self, api_token):
        self.api = BitriseApiFetcher(api_token)
        self.converter = RawDataConverter()

    def available_projects(self):
        endpoint = f"{BITRISE_API_URL}/apps"
        raw_data = self.api.get(endpoint)
        return self.converter.projects_from(raw_data)

    def builds_for_project(self, project, starting=None, ending=None):
        endpoint = f"{BITRISE_API_URL}/apps/{project.slug}/builds"
        raw_data = self.api.get(endpoint, starting, ending)
        return self.converter.builds_from(raw_data, project)


class BitriseApiFetcher(object):
    def __init__(self, api_token):
        self.auth = {"Authorization": api_token}

    def get(self, endpoint, starting=None, ending=None):
        results = []
        next = FIRST_PAGE

        params = {}

        if starting:
            params["after"] = self.__unixtime(starting)

        if ending:
            params["before"] = self.__unixtime(ending)

        while next != NO_MORE_PAGES:

            if next != FIRST_PAGE:
                params["next"] = next

            fetched, next_page = self.__get_page(endpoint, params)
            results.extend(fetched)
            next = next_page

        return results

    def __get_page(self, endpoint, args=None):
        response = requests.get(endpoint, headers=self.auth, params=args)

        if response.status_code == requests.codes.ok:
            data = response.json()["data"]
            paging = response.json()["paging"]
            next = paging["next"] if "next" in paging.keys() else NO_MORE_PAGES
            return data, next
        else:
            cause = ErrorCause.NetworkingInfrastructure
            message = f"""
            Error when retriving data from : {endpoint}
            Status = {response.status_code}
            """
            raise BitriseReportsError(cause, message)

    def __unixtime(self, datetime):
        return int(time.mktime(datetime.timetuple()))


class RawDataConverter(object):
    def projects_from(self, json):
        def conversion(json, project):
            return [BitriseProject(item["title"], item["slug"]) for item in json]

        return self.__safely_convert(conversion, json)

    def builds_from(self, json, project):
        def conversion(json, project):
            finished_builds = list(filter(lambda raw: raw["finished_at"] is not None, json))
            return [self.build_from(item, project) for item in finished_builds]

        return self.__safely_convert(conversion, json, project)

    def __safely_convert(self, callable, json, project=None):
        try:
            return callable(json, project)
        except Exception as e:
            print(e)
            cause = ErrorCause.DataConversion
            message = "Could not parse/convert information from builds"
            raise BitriseReportsError(cause, message)

    def build_from(self, json, project):
        machine = self.machine_from(json["machine_type_id"], json["stack_identifier"])
        workflow = self.workflow_from(json["triggered_workflow"], project)
        minutes = self.minutes_from(
            json["triggered_at"], json["started_on_worker_at"], json["finished_at"]
        )

        status = self.status_from(json["status"])
        return BitriseBuild(project, machine, workflow, minutes, status)

    def machine_from(self, machine_type_id, stack_identifier):
        size = MachineSize(machine_type_id)
        stack = BuildStack("macos" if "osx" in stack_identifier else "linux")
        return BuildMachine(f"{stack.value}.{size.value}", size, stack)

    def workflow_from(self, triggered_workflow, bitrise_project):
        return BitriseWorkflow(triggered_workflow)

    def minutes_from(self, triggered_at, started_at, finished_at):
        triggered = self.__dt(triggered_at)
        started = self.__dt(started_at) if started_at is not None else triggered
        finished = self.__dt(finished_at)
        queued = self.__aproximate_minutes(started - triggered)
        building = self.__aproximate_minutes(finished - started)
        total = self.__aproximate_minutes(finished - triggered)
        return BuildMinutes(queued, building, total)

    def status_from(self, status):
        return ExecutionStatus(status) if status in range(1, 3) else ExecutionStatus.other

    def __dt(self, timestamp):
        return datetime.fromisoformat(timestamp.replace("Z", ""))

    def __aproximate_minutes(self, diff):
        return ceil((diff.days * 24 * 60) + (diff.seconds / 60))



# di.py

from . import cli
from .app import Application
from .bitrise import Bitrise
from .middleware import ProjectFinder, BuildsAnalyser
from .metrics import MetricsCruncher
from .reporting import MetricsReporter
from .models import EvaluationCriteria


def inject(token, app, starting, ending, velocity, statuses, report):

    bitrise = Bitrise(token)
    finder = ProjectFinder(bitrise)
    analyser = BuildsAnalyser(bitrise, MetricsCruncher())

    criteria = EvaluationCriteria(
        cli.validated_app(app),
        cli.validated_date(starting, include_hours=False),
        cli.validated_date(ending, include_hours=True),
    )

    reporter = MetricsReporter(criteria, velocity, statuses, cli.validated_report(report))

    return Application(finder, analyser, reporter, criteria)



# middleware.py

from .errors import ErrorCause, BitriseReportsError

import logging


class ProjectFinder(object):
    def __init__(self, bitrise):
        self.bitrise = bitrise

    def find(self, name):
        projects = self.bitrise.available_projects()
        target = next(filter(lambda project: project.id == name, projects), None)

        if not target:
            logging.error("Cannot locate an app with name: {name}")
            logging.error("Available projects for this user")
            logging.error(projects)
            cause = ErrorCause.MiddlewareOrchestration
            message = f"{name} not present among projects you have access"
            raise BitriseReportsError(cause, message)

        return target


class BuildsAnalyser(object):
    def __init__(self, bitrise, cruncher):
        self.bitrise = bitrise
        self.cruncher = cruncher

    def analyse(self, project, starting=None, ending=None):
        builds = self.bitrise.builds_for_project(project, starting, ending)

        if not builds:
            logging.error(f"Cannot locate find builds for project : {project.id}")
            cause = f"{project.id} has no builds yet"
            message = f"{project.id} has no builds yet"
            raise BitriseReportsError(cause, message)

        breakdowns = [
            self.cruncher.breakdown_per_project(builds),
            self.cruncher.breakdown_per_machine(builds),
            self.cruncher.breakdown_per_workflow(builds),
        ]

        return breakdowns



# app.py

from dataclasses import astuple


class Application(object):
    def __init__(self, finder, analyser, reporter, criteria):
        self.project_finder = finder
        self.project_analyser = analyser
        self.results_reporter = reporter
        self.execution_criteria = criteria

    def execute(self):
        bitrise_app, starting, ending = astuple(self.execution_criteria)

        self.results_reporter.started(bitrise_app)

        project = self.project_finder.find(bitrise_app)
        self.results_reporter.feedback(project)

        breakdowns = self.project_analyser.analyse(project, starting, ending)
        self.results_reporter.report(breakdowns)



# reporting.py

from rich.console import Console
from rich.table import Table
from openpyxl import Workbook

import json
import os


class MetricsReporter:
    def __init__(self, criteria, velocity, statuses, strategy, console=Console()):
        self.contextual_delegate = ContextReporter(criteria, velocity, statuses, console)
        self.delegates = {
            "stdout": StdoutReporter(criteria, velocity, statuses, console),
            "json": JsonReporter(criteria, velocity, statuses, console),
            "excel": ExcelReporter(criteria, velocity, statuses, console),
        }
        self.output_delegate = self.__find_delegate(strategy)

    def started(self, app):
        self.contextual_delegate.started(app)

    def feedback(self, project):
        self.contextual_delegate.feedback(project)

    def report(self, breakdowns):
        self.output_delegate.report(breakdowns)

    def __find_delegate(self, strategy):
        return self.delegates[strategy]


class ContextReporter:
    def __init__(self, criteria, velocity, statuses, console):
        self.criteria = criteria
        self.console = console
        self.velocity = velocity
        self.statuses = statuses

    def started(self, app):
        self.__print("\nAnalysing", app, "cyan")

    def feedback(self, project):
        self.__print("\nFound slug", project.slug, "cyan")
        self.__print("Starting", self.__format_date(self.criteria.starting_at), "pink")
        self.__print("Ending  ", self.__format_date(self.criteria.ending_at), "pink")

    def __print(self, prefix, content, color):
        printer = self.console
        printer.print(f"{prefix} → [bold {color}]{content}[/bold {color}]")

    def __format_date(self, datetime):
        return f"{datetime:%A, %d %B (%Y)}"


class StdoutReporter(ContextReporter):
    def report(self, breakdowns):
        printer = self.console
        printer.print("\nBuilds processed with success!")
        printer.print(
            "[bold green]NOTE[/bold green] : build times in [bold cyan] minutes [/bold cyan] "
        )

        for breakdown in breakdowns:
            self.report_breakdown(breakdown)

    def report_breakdown(self, breakdown):
        printer = self.console
        printer.print(f"\n[bold green]{breakdown.name}[/bold green]")

        table = Table(show_header=True, header_style="bold magenta")
        table.pad_edge = False
        table.add_column("Name")
        table.add_column("Builds", justify="right")
        table.add_column("Queued time", justify="right")
        table.add_column("Building time", justify="right")
        table.add_column("Total time", justify="right")

        if self.statuses:
            table.add_column("Successes", justify="right")
            table.add_column("Failures", justify="right")

        if self.velocity:
            table.add_column("Credits Estimation", justify="right")

        sorted_by_total = sorted(
            breakdown.details.items(), key=lambda kv: kv[1].count, reverse=True
        )

        for entry, value in sorted_by_total:
            rows = [
                entry.id,
                f"{value.count}",
                f"{value.queued}",
                f"{value.building}",
                f"{value.total}",
            ]

            if self.statuses:
                rows.append(f"{value.successes}")
                rows.append(f"{value.failures}")

            if self.velocity:
                rows.append(f"{value.credits}")

            table.add_row(*rows)

        printer.print(table)


class JsonReporter(ContextReporter):
    def __init__(
        self,
        criteria,
        velocity,
        statuses,
        console,
        filename="bitrise-metrics.json",
        folder=os.getcwd(),
    ):
        super(JsonReporter, self).__init__(criteria, velocity, statuses, console)
        self.filename = filename
        self.folder = folder

    def report(self, breakdowns):
        data = [self.__process(item) for item in breakdowns]
        path = f"{self.folder}/{self.filename}"

        with open(path, "w") as writer:
            json.dump(data, writer, indent=4)
            self.console.print(f"\nWrote results at [bold green]{path}[/bold green]")

    def __process(self, breakdown):
        flattened = {"description": breakdown.name, "details": []}

        sorted_by_total = sorted(
            breakdown.details.items(), key=lambda kv: kv[1].count, reverse=True
        )

        for analysed, value in sorted_by_total:
            entry = {
                "name": analysed.id,
                "count": value.count,
                "queued": value.queued,
                "building": value.building,
                "total": value.total,
            }

            if self.statuses:
                entry["successes"] = value.successes
                entry["failures"] = value.failures

            if self.velocity:
                entry["credits"] = value.total.credits

            flattened["details"].append(entry)

        return flattened


class ExcelReporter(ContextReporter):
    def report(self, breakdowns):
        excel_file = "bitrise-metrics.xlsx"
        path = f"{os.getcwd()}/{excel_file}"

        workbook = Workbook()
        sheet = workbook.active

        sheet.column_dimensions["A"].width = 25

        for column in ["A", "B", "C", "D", "E", "F", "G", "H"]:
            sheet.column_dimensions[column].auto_size = True
            sheet.column_dimensions[column].bestFit = True

        line = 1

        for breakdown in breakdowns:
            sheet[f"A{line}"] = breakdown.name
            line = line + 1

            sorted_by_total = sorted(
                breakdown.details.items(), key=lambda kv: kv[1].count, reverse=True
            )

            sheet[f"A{line}"] = "Name"
            sheet[f"B{line}"] = "Builds"
            sheet[f"C{line}"] = "Queued time"
            sheet[f"D{line}"] = "Building time"
            sheet[f"E{line}"] = "Total time"

            velocity_column = "H" if self.statuses else "F"

            if self.statuses:
                sheet[f"F{line}"] = "Build successes"
                sheet[f"G{line}"] = "Build failures"

            if self.velocity:
                sheet[f"{velocity_column}{line}"] = "Credits estimation"

            for entry, value in sorted_by_total:
                line = line + 1

                sheet[f"A{line}"] = entry.id
                sheet[f"B{line}"] = value.count
                sheet[f"C{line}"] = value.queued
                sheet[f"D{line}"] = value.building
                sheet[f"E{line}"] = value.total

                if self.statuses:
                    sheet[f"F{line}"] = value.successes
                    sheet[f"G{line}"] = value.failures

                if self.velocity:
                    sheet[f"{velocity_column}{line}"] = value.credits

            line = line + 2

        workbook.save(filename=excel_file)
        self.console.print(f"\nWrote results at [bold green]{path}[/bold green]")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bitrise_reports']

package_data = \
{'': ['*']}

install_requires = \
['click==7.1.2',
 'openpyxl==3.0.7',
 'pretty-errors==1.2.20',
 'python-dateutil==2.8.1',
 'requests==2.25.1',
 'rich==10.1.0']

entry_points = \
{'console_scripts': ['bitrise-reports = bitrise_reports:main']}

setup_kwargs = {
    'name': 'bitrise-reports',
    'version': '0.0.3',
    'description': 'The missing tool to extract reports about projects you build on Bitrise',
    'long_description': '# Bitrise Reports\n\n[![Flake8](https://img.shields.io/badge/codestyle-flake8-yellow)](https://flake8.pycqa.org/en/latest/)\n[![Black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Quality](https://api.codeclimate.com/v1/badges/a9fe25bd995710be45d2/maintainability)](https://codeclimate.com/github/dotanuki-labs/bitrise-reports/maintainability)\n[![Coverage](https://codecov.io/gh/dotanuki-labs/bitrise-reports/branch/main/graph/badge.svg)](https://codecov.io/gh/dotanuki-labs/bitrise-reports)\n[![PyPI](https://img.shields.io/pypi/v/bitrise-reports)](https://pypi.org/project/bitrise-reports/)\n[![Main](https://github.com/dotanuki-labs/bitrise-reports/workflows/Main/badge.svg)](https://github.com/dotanuki-labs/bitrise-reports/actions?query=workflow%3AMain)\n[![License](https://img.shields.io/github/license/dotanuki-labs/bitrise-reports)](https://choosealicense.com/licenses/mit)\n\n## What\n\nA simple cruncher for numbers derived from builds you run on [Bitrise CI](https://www.bitrise.io/). Useful if you are in charge of managing infrastructure capacity related to Bitrise, like detecting/reporting anomalies, evaluating queues impact and so on.\n\nMain features:\n\n- Backed by [Bitrise REST API](https://api-docs.bitrise.io/) under the hood\n- Computes queued, running and total execution times for all builds found in the target time frame\n- Breakdown numbers per machine type and also per Workflow\n- Supports emulation of consumed [Bitrise Velocity credits](https://www.bitrise.io/velocity-plan) (for Enterprise customers)\n- Report types : CLI (stdout), JSON and Excel spreadsheet\n\n## Installing\n\nThis tool requires Python, supporting versions 3.8.x and 3.9.x.\n\nInstall `bitrise-reports` with [pip](https://pypi.org/project/pip/)\n\n```bash\n→ pip install bitrise-reports\n```\n\n## Using\n\nLet\'s say you want analyse numbers for the project `my-app`, learning from\nbuilds that ran during February of 2021. You\'ll firstly need a\n[Bitrise Personal Access Token](https://devcenter.bitrise.io/api/authentication/) for\nthat. Note you must be a member in the project you want to analyse.\n\nBy running\n\n```bash\n→ bitrise-reports \\\n    --token=$BITRISE_PAT_TOKEN \\\n    --app=my-app \\\n    --starting=2021-02-01 \\\n    --ending=2021-02-28\n```\nyou should get something like that on your CLI\n\n![](.github/assets/showcase-cli.png)\n\nThe complete list of CLI options:\n\n| Option   | Details                                           | Required  |\n|----------|---------------------------------------------------|-----------|\n| token    | Personal access token for Bitrise API             | Yes       |\n| app      | The title of your app in Bitrise                  | Yes       |\n| starting | Starting date in the target time frame            | Yes       |\n| ending   | Ending date in the target time frame              | Yes       |\n| report   | The style of report you want                      | No        |\n| statuses | Estimate successes and failures related to builds | No        |\n| velocity | Estimate Bitrise Velocity credits consumed        | No        |\n\n\nwhere\n\n- `starting` and `ending` follow **YYYY-MM-DD** convention\n- `report` accepts **stdout** (default), **json** or **excel**\n- `velocity` is a CLI flag\n- `statuses` is a CLI flag\n\nFor instance, if you want an Excel spreadsheet - instead of the fancy Terminal UI from the previous example -\nand also an estimation of Velocity usage, you can run\n\n```bash\n→ bitrise-reports \\\n    --token=$BITRISE_PAT_TOKEN \\\n    --app=my-app \\\n    --starting=2021-02-01 \\\n    --ending=2021-02-28 \\\n    --report=excel \\\n    --velocity\n```\n\nand the output file `bitrise-metrics.xlsx` will be available in the same folder.\n\n![](.github/assets/showcase-excel.png)\n\n## Contributing\n\nIf you want to contribute with this project\n\n- Check the [contribution guidelines](https://github.com/dotanuki-labs/.github/blob/main/CONTRIBUTING.md)\n- Ensure you have Python 3.8.+ installed. I recommend [Pyenv](https://github.com/pyenv/pyenv) for that.\n- Ensure you have [Poetry](https://python-poetry.org/) installed\n- Prepare your environment with [Flake8](https://pypi.org/project/flake8/), [Black](https://pypi.org/project/black/) and [Bandit](https://pypi.org/project/bandit/)\n\n```bash\n→ make setup\n```\n\n- Code you changes\n- Make sure you have a green build\n\n```bash\n→  make inspect test\n```\n\n- Submit your PR 🔥\n\n## Author\n\n- Coded by Ubiratan Soares (follow me on [Twitter](https://twitter.com/ubiratanfsoares))\n\n## License\n\n```\nThe MIT License (MIT)\n\nCopyright (c) 2021 Dotanuki Labs\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of\nthis software and associated documentation files (the "Software"), to deal in\nthe Software without restriction, including without limitation the rights to\nuse, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of\nthe Software, and to permit persons to whom the Software is furnished to do so,\nsubject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS\nFOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR\nCOPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER\nIN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN\nCONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\n```\n',
    'author': 'Ubiratan Soares',
    'author_email': 'ubiratanfsoares@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dotanuki-labs/bitrise-reports',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
